#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "util.h"

irr::IrrlichtDevice *device;
video::IVideoDriver* driver;
scene::ISceneManager* smgr;
gui::IGUIEnvironment* env;
irr::ITimer* timer;

bool keys[KEY_KEY_CODES_COUNT];
bool mouseLeft;
bool mouseRight;
bool mouseMiddle;
f32 mouseWheel;

static int tmp_fps=0;
static int fps=100;
static double last_100ms=0;
static int window_size_x;
static int window_size_y;
static bool is_fullscreen;
static int display_bits;
static video::E_DRIVER_TYPE driver_type;


static void initIrrlichtElements(void)
{   
	srand(time(NULL));
    
	if (is_fullscreen)
		printf("Lauching application %dx%d in fullscreen mode\n",window_size_x,window_size_y,(int)is_fullscreen);
	else
	        printf("Lauching application %dx%d in windowed mode\n",window_size_x,window_size_y,(int)is_fullscreen);

	if (device == 0)
	        util_PrintCriticalError("Could not create selected driver.");
    
	driver = device->getVideoDriver();
	if (driver == 0)
	     util_PrintCriticalError("Could not create VideoDriver.");	
	smgr = device->getSceneManager();
	if (smgr == 0)
	     util_PrintCriticalError("Could not create SceneManager.");
	env = device->getGUIEnvironment();
	if (env == 0)
	     util_PrintCriticalError("Could not create GUIEnvironment.");
	
	timer=device->getTimer();
	last_100ms=timer->getTime();
	
	device->getCursorControl()->setVisible(false);
}

void util_InitNullDevice(void)
{
    device = createDevice(EDT_NULL,core::dimension2d<s32>(640,480),16, true);
}    

void util_DropDevice(void)
{
	device->drop();
}

void util_Init(IEventReceiver *receiver,int window_size_x_,int window_size_y_,
    bool is_fullscreen_,int display_bits_,video::E_DRIVER_TYPE driver_type_)
{
	window_size_x=window_size_x_;
	window_size_y=window_size_y_;
	is_fullscreen=is_fullscreen_;
	display_bits=display_bits_;
	driver_type=driver_type_;
	
	device = createDevice(driver_type,
	        core::dimension2d<s32>((int)window_size_x,(int)window_size_y),
		display_bits, is_fullscreen);

	device->setEventReceiver(receiver);
	
 	initIrrlichtElements();
}

void util_PrintError(const char *text)
{ 
    fprintf(stderr,"ERROR : %s \n",text);
    core::stringw strtmp = L"";
    strtmp += "ERROR : ";
    strtmp += text;
    strtmp += ". PRESS RETURN KEY TO CONTINUE";
    IGUIStaticText *static_text=env->addStaticText(strtmp.c_str(),
		rect<s32>(10,10,util_GetWindowSizeX()-10,util_GetWindowSizeY()-10),
		false, // border?
		true // wordwrap?
		);
	
	while(!keys[KEY_RETURN] && device->run() && driver)
	{
	    driver->beginScene(true, true, video::SColor(255,255,255,255));
        env->drawAll();
        driver->endScene();
        if (keys[KEY_ESCAPE])
             util_QuitApplication();
    }    
}   

void util_PrintCriticalError(const char *text)
{  
    fprintf(stderr,"CRITICAL ERROR : %s \n",text);
    system("PAUSE");
    exit(EXIT_FAILURE);
}   

void util_CalcFPS()
{
    
	//calcul du fps
	tmp_fps++;
	if (timer->getTime()>=last_100ms+100)
	{
	     last_100ms=timer->getTime();
	     fps=tmp_fps*10;
	     tmp_fps=0;
    }
}    

double util_Random(void)
{
    int max=10;
  float multi_rand=(float) rand() * max;
  float r=multi_rand / (float) RAND_MAX;
  //fprintf(stderr,"\n%lf\n",(double)r/10);
  return ((double)(r/10));
}

int util_GetFPS(void)
{
    if (fps==0) return 1;
    return fps;
}    

bool util_WaitForReturnKey(core::stringw title)
{
    while(!keys[KEY_RETURN])
    {
		driver->beginScene(true, true, video::SColor(0,0,0,0));
		smgr->drawAll();
		env->drawAll();
		driver->endScene();
		util_CalcFPS();

        if (!device->run() || !driver)
            util_QuitApplication();
        if (keys[KEY_ESCAPE])
            return false;
    }   
    return true;
}    

void util_QuitApplication()
{
	device->drop();
	exit(0);
}

const char *util_TestExistFile(const char *file_path)
{
    IFileSystem *filesystem=device->getFileSystem();
    if (!filesystem->existFile(file_path))
    {
        fprintf(stderr,"%s : ",file_path);
        util_PrintCriticalError("Can't find this file");
    }   
    return file_path;
}    

int util_GetWindowSizeX(void) {return window_size_x;}
int util_GetWindowSizeY(void) {return window_size_y;}


video::E_DRIVER_TYPE util_GetDriverType(int num)
{
    switch(num)
    {
        case 0:return video::EDT_NULL;
        case 1:return video::EDT_SOFTWARE;
        case 2:return video::EDT_OPENGL;
        case 3:return video::EDT_DIRECT3D8;
        case 4:return video::EDT_DIRECT3D9;
        case 5:
#ifdef WIN32
            return video::EDT_DIRECT3D8;
#else
            return video::EDT_OPENGL;
#endif

        case 6:
#ifdef WIN32
            return video::EDT_DIRECT3D9;
#else
            return video::EDT_OPENGL;
#endif
        default:util_PrintCriticalError("util_GetDriverType : unknown value");
    }   
}    

int util_GetDriverNum(video::E_DRIVER_TYPE type)
{
    switch(type)
    {
        case video::EDT_NULL:return 0;
        case video::EDT_SOFTWARE:return 1;
        case video::EDT_OPENGL:return 2;
        case video::EDT_DIRECT3D8:return 3;
        case video::EDT_DIRECT3D9:return 4;
        default:util_PrintCriticalError("util_GetDriverNum : unknown value");
    }     
}    

bool util_IsKeyPressed(void)
{
	for (int i=0;i<KEY_KEY_CODES_COUNT;i++)
		if (keys[i])
			return true;
	
	return false;
}


void util_Fade(video::SColor color,int time)
{
	gui::IGUIInOutFader* fader = device->getGUIEnvironment()->addInOutFader();
	fader->setColor(color);
	fader->fadeIn(time);
}

double util_GetDistance(vector3df pos1,vector3df pos2)
{
    return sqrt(pow(pos1.X-pos2.X,2)+pow(pos1.Y-pos2.Y,2)+pow(pos1.Z-pos2.Z,2));
}

int util_GetTimerTime(void)
{
    return timer->getTime();
}
